/*
 * Purple - Internet Messaging Library
 * Copyright (C) Pidgin Developers <devel@pidgin.im>
 *
 * Purple is the legal property of its developers, whose names are too numerous
 * to list here. Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this library; if not, see <https://www.gnu.org/licenses/>.
 */

#if !defined(PURPLE_GLOBAL_HEADER_INSIDE) && !defined(PURPLE_COMPILATION)
# error "only <purple.h> may be included directly"
#endif

#ifndef PURPLE_CONVERSATION_MANAGER_H
#define PURPLE_CONVERSATION_MANAGER_H

#include <glib.h>

#include "purplecontact.h"
#include "purpleconversation.h"
#include "purpleversion.h"

G_BEGIN_DECLS

#define PURPLE_TYPE_CONVERSATION_MANAGER (purple_conversation_manager_get_type())

/**
 * PurpleConversationManager:
 *
 * #PurpleConversationManager keeps track of all #PurpleConversation's inside
 * of libpurple and allows searching of them.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
G_DECLARE_FINAL_TYPE(PurpleConversationManager, purple_conversation_manager,
                     PURPLE, CONVERSATION_MANAGER, GObject)

/**
 * purple_conversation_manager_add:
 * @manager: The #PurpleConversationManager instance.
 * @conversation: The #PurpleConversation to add.
 *
 * Adds @conversation to @manager.
 *
 * Returns: %TRUE if @conversation was not yet added.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_conversation_manager_add(PurpleConversationManager *manager, PurpleConversation *conversation);

/**
 * purple_conversation_manager_new:
 * @filename: (nullable): The filename to serialize to.
 *
 * Creates a new conversation manager that will be backed by @filename.
 *
 * This is typically only called by user interfaces in
 * [vfunc@Ui.get_conversation_manager].
 *
 * If you need a [class@ConversationManager] instance, you probably want to use
 * [func@ConversationManager.get_default].
 *
 * Returns: (transfer full): The new instance.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
PurpleConversationManager *purple_conversation_manager_new(const char *filename);

/**
 * purple_conversation_manager_get_default:
 *
 * Gets the default instance of #PurpleConversationManager. This instance
 * can be used for any of the API including connecting to signals.
 *
 * Returns: (transfer none): The default #PurpleConversationManager instance.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
PurpleConversationManager *purple_conversation_manager_get_default(void);

/**
 * purple_conversation_manager_get_default_as_model:
 *
 * Gets the default instance of #PurpleConversationManager but cast to a
 * [iface@Gio.ListModel].
 *
 * Returns: (transfer none): The instance cast to a list model.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
GListModel *purple_conversation_manager_get_default_as_model(void);

/**
 * purple_conversation_manager_get_filename:
 * @manager: The instance.
 *
 * Gets the filename the @manager should be serializing itself to.
 *
 * Returns: (transfer none) (nullable): The filename.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
const char *purple_conversation_manager_get_filename(PurpleConversationManager *manager);

/**
 * purple_conversation_manager_get_all:
 * @manager: The #PurpleConversationManager instance.
 *
 * Gets a list of all conversations that @manager knows about.
 *
 * Returns: (transfer container) (element-type PurpleConversation): A list of
 *          all of the conversations that @manager knows about.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
GList *purple_conversation_manager_get_all(PurpleConversationManager *manager);

/**
 * purple_conversation_manager_find_dm:
 * @manager: The instance.
 * @contact: The contact.
 *
 * Looks for a direct message with @contact and returns it if found.
 *
 * Returns: (transfer none) (nullable): The conversation if found, otherwise
 *          %NULL.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
PurpleConversation *purple_conversation_manager_find_dm(PurpleConversationManager *manager, PurpleContact *contact);

/**
 * purple_conversation_manager_find_with_id:
 * @manager: The instance.
 * @account: The account the conversation belongs to.
 * @id: The identifier of the conversation.
 *
 * Looks for a conversation belonging to @account with an id of @id.
 *
 * Returns: (transfer none) (nullable): The [class@PurpleConversation] if
 *          found, otherwise %NULL.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
PurpleConversation *purple_conversation_manager_find_with_id(PurpleConversationManager *manager, PurpleAccount *account, const char *id);

/**
 * purple_conversation_manager_remove:
 * @manager: The #PurpleConversationManager instance.
 * @conversation: The #PurpleConversation to remove.
 *
 * Removes @conversation from @manager.
 *
 * Returns: %TRUE if @conversation was found and removed.
 *
 * Since: 3.0
 */
PURPLE_AVAILABLE_IN_3_0
gboolean purple_conversation_manager_remove(PurpleConversationManager *manager, PurpleConversation *conversation);

G_END_DECLS

#endif /* PURPLE_CONVERSATION_MANAGER_H */
